package CTL.CCompat;

import ReflWrap.*;
import java.lang.reflect.*;
import java.util.*;

/** Compiler which converts CTL4j interfaces to a C++ component CI */
public class CTLcc
{
	/** Common header for all CIs
	 * @return Header
	 */
	private static String head ()
	{
		return "/* Generated by CTLcc, written by Boris Buegling */\n\n" +
			"#include <ctl.h>\n\n";
	}

	/** Header guard for a specific class
	 * @param cinfo ClassInfo object
	 * @return Header guard
	 */
	private static String headerGuard (ClassInfo cinfo)
	{
		StringBuffer buf = new StringBuffer();
		StringTokenizer tok = new StringTokenizer(cinfo.fqcn(), ".");

		buf.append("__");
		while (tok.hasMoreTokens())
			buf.append(tok.nextToken()+"_");
			
		return buf.toString();
	}

	/** Conversion from Java types to CTL types
	 * @param typ String representation of a Java type
	 * @return String representation of the equivalent CTL type
	 */
	private static String type2RI (String typ)
	{
		String type = Refl.lastToken(typ, " ");
		int len = type.length();
		
		if (type.substring(len-2, len).equals("[]"))
			return "array<"+type2RI(type.substring(0, len-2))+">";
		if (type.equals("boolean"))
			return "bool";
		if (type.equals("float"))
			return "ctl::real4";
		if (type.equals("double"))
			return "ctl::real8";
		if (type.equals("short"))
			return "ctl::int2";
		if (type.equals("int") || type.equals("Integer"))
			return "ctl::int4";
		if (type.equals("long"))
			return "ctl::int8";
		if (type.equals("java.lang.String"))
			return "std::string";
		if (type.equals("void"))
			return type;
		
		type = Refl.lastToken(type, ".");
		if (type.equals("AnyObj"))
			return "ctl::any";

		System.err.println("WARNING: This class uses the user-defined type '"+
			type+"'.");
		return type;
	}

	/** Generates a component interface (CI) for a specific class
	 * @param cinfo Class to use
	 * @return String Complete component interface
	 */
	private static String classRI (ClassInfo cinfo)
	{
		StringBuffer buf = new StringBuffer();
		
		buf.append("#define CTL_Class "+cinfo.name()+"\n");
		buf.append("  #include CTL_ClassBegin\n\n");
		
		// Constructors
		Constructor[] foo = cinfo.constructors();
		for (int i=0;i<foo.length;i++)
		{
			ConstructInfo moo = new ConstructInfo(foo[i]);

			buf.append("    #define CTL_Constructor"+i+" (");
			for (int j=0;j<moo.params().length;j++)
			{
				buf.append(j==0 ? "" : ", ");
				if (Refl.hasAnot("@CTL.const_()", moo.getParamAnot(j)))
					buf.append("const ");
				buf.append(type2RI(moo.params()[j].toString()));
			}
			buf.append("), "+moo.params().length+"\n");
		}
		
		// Methods
		Method[] boo = cinfo.meth_here();
		for (int i=0;i<boo.length;i++)
		{
			MethodInfo moo = new MethodInfo(boo[i]);
			
			buf.append("    #define CTL_");
			if (moo.isStatic())
				buf.append("Static");
			buf.append("Method"+(i+1)+" ");
			buf.append(type2RI(moo.returns().toString())+", ");
			buf.append(moo.name()+", (");
			for (int j=0;j<moo.params().length;j++)
			{
				buf.append(j==0 ? "" : ", ");
				if (Refl.hasAnot("@CTL.Annotate.const_()", moo.getParamAnot(j)))
					buf.append("const ");
				buf.append(type2RI(moo.params()[j].toString()));
			}
			buf.append("), "+moo.params().length+"\n");
		}
		
		buf.append("\n  #include CTL_ClassEnd\n");
		
		return buf.toString();
	}
	
	/** Convenience function
	 * @param cinfo Class to use
	 * @return CI with common header and header guards
	 */
	private static String spitOutRI (ClassInfo cinfo)
	{
		String guard = headerGuard(cinfo);
		StringBuffer buf = new StringBuffer();
		
		buf.append(head());
		buf.append("#ifndef "+guard+"\n");
		buf.append("#define "+guard+"\n\n");
		buf.append(classRI(cinfo));
		buf.append("#endif //"+guard+"\n");
		buf.append("// vim: ft=cpp\n");
		
		return buf.toString();
	}
	
	public static void main (String[] args)
	{
		if (args.length<1)
		{
			System.err.println("No class specified.");
			System.exit(1);
		}

		try
		{
			System.out.print(spitOutRI(new ClassInfo(args[0])));
		}
		catch (ClassNotFoundException e)
		{
			System.err.println("Class "+args[0]+" not found.");
			System.exit(2);
		}
	}
}
