package CTL;

import ReflWrap.*;

import java.io.*;
import java.util.*;

/** Filter-based logging class */
public class Logger
{
	/** Error */
	public static final int ERR = 2;
	/** Warning */
	public static final int WARN = 3;
	/** Information */
	public static final int INFO = 4;
	/** Debugging information, level 0 */
	public static final int DBG = 5;
	/** Debugging information, level 1 */
	public static final int DBG1 = 6;
	/** Debugging information, level 2 */
	public static final int DBG2 = 7;
	/** All information */
	public static final int ALL = Integer.MAX_VALUE;
	
	/** The file to write to */
	private FileWriter file;
	/** Filter level of this Logger (default: anything equally or more 
	 * important than warning) */
	private static int loglevel = WARN;

	// TODO: document
	private Logger (String fname)
	{
		this(fname, true);
	}

	/** Generate a new Logger with default filter
	 * @param fname Logfile
	 * @param mangle Whether or not to modify the fname
	 */
	private Logger (String fname, boolean mangle)
	{
		if (!Globals.debugMode)
			return;

		try
		{
			if (fname.equals("stdout"))
			{
				mangle = false;
				fname = "/proc/self/fd/1";
			}

			if (fname.equals("stderr"))
			{
				mangle = false;
				fname = "/proc/self/fd/2";
			}

			if (mangle)
			{
				String stamp = "";
				if (Globals.LogTimeStamp)
					stamp = "-"+gettimeofday(true);
				if (System.getProperty("os.name").startsWith("Windows"))
				{
					StringTokenizer tok = new StringTokenizer(System.getenv("TMP"), "\\");
					StringBuffer tmp = new StringBuffer();
					while (tok.hasMoreTokens())
						tmp.append(tok.nextToken()+"/");
					fname = ByteCode.sed(fname, "/tmp/", tmp.toString());
				}
				fname = fname+stamp+".log";
			}

			file = new FileWriter(fname);
		}
		catch (IOException e)
		{
			System.err.println("CTL.Logger: "+e.getMessage());
		}
	}

	/** Generate a new Logger with user-defined filter treshold
	 * @param fname Logfile
	 * @param loglevel Filter treshold
	 */
	public Logger (String fname, int loglevel, boolean mangle)
	{
		this(fname, mangle);
		Logger.loglevel = loglevel;
	}

	// TODO: document
	public Logger (String fname, int loglevel)
	{
		this(fname, loglevel, true);
	}

	/** Close the underlying file */
	public void close ()
	{
		try
		{
			file.close();
		}
		catch (IOException e)
		{
			System.err.println("Logging error.");
		}
	}

	/** Write a message to the logfile
	 * @param lvl Importance of this message, if equally or more important
	 * than the filter treshold, it will be logged
	 * @param msg Log message
	 */
	public void msg (int lvl, String msg)
	{
		if (!Globals.debugMode || file == null)
			return;

		try
		{
			if (Globals.LogScope)
			{
				Throwable ex = new Throwable();
				for (int i=0; i<Globals.LogScopeDepth; i++)
				{
					StackTraceElement head = ex.getStackTrace()[i+1];
					msg = head.getClassName()+"."+head.getMethodName()+" :: "+msg;
				}
			}

			if (lvl<=loglevel)
			{
				file.write(gettimeofday(false)+"  "+msg+
					System.getProperty("line.separator"));
				file.flush();
			}
		}
		catch (IOException e)
		{
			System.err.println("Logging error.");
		}
	}

	/** Write a message to the logfile
	 * @param msg Log message
	 */
	public void msg (String msg)
	{
		msg(Logger.INFO, msg);
	}

	/** Determine current time 
	 * @return Pretty-printed string containing the time of day
	 */
	private static String gettimeofday (boolean foo)
	{
		SimpleTimeZone pdt = new SimpleTimeZone(1 * 60 * 60 * 1000, 
			"Europe/Berlin");
		Calendar cal = new GregorianCalendar(pdt);
		Date date = new Date();
		cal.setTime(date);

		StringBuffer buf = new StringBuffer();
		try
		{
			Formatter fmt = new Formatter(buf);
			if (foo)
				fmt.format("%d%d%d", cal.get(Calendar.HOUR_OF_DAY),
					cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND));
			else	
				fmt.format("%2d:%2d:%2d", cal.get(Calendar.HOUR_OF_DAY),
					cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND));
		}
		catch (Exception e) // Hack for GNU Classpath compatibility
		{
			if (foo)
				buf.append(""+cal.get(Calendar.HOUR_OF_DAY)+cal.get(Calendar.MINUTE)+
					cal.get(Calendar.SECOND));
			else
				buf.append(cal.get(Calendar.HOUR_OF_DAY)+":"+cal.get(Calendar.MINUTE)+
					":"+cal.get(Calendar.SECOND));
		}
		return buf.toString();
	}

	public static void main (String[] args)
	{
		System.out.println(gettimeofday(true));
	}
}
